#!/usr/bin/perl
#
#  based on ToasterView 0.2, by Ben Rockwood (benr@cuddletech.com)
#
#  License: GPL (http://www.gnu.org/licenses/gpl.txt)
#  Description: A simple tool that generates a web page summary
#               of a NetApp Filer status using SNMP
#  Requirements: Perl
#################################################################################
#
# Version : 1.0 = initial release
#
# Version : 1.1 = changed gmtime() to localtime()
#
# Version : 1.2 = corrected gauge_width_RED calc
#
# Version : 1.3 = made config external
#
# Version : 2.0 = redirected debug output to STDERR
#                 added logic to read filer list from config file
#                 changed command line arguments to get config filename from it
#                   and no longer filer and optional community
#                 html output is send to a file and no longer to STDOUT
#
# Version : 2.1 = redirected logging to STDOUT
#
# Version : 2.2 = changed name from FilerView to FilerStatus
#                 NetApp has already FilerView name
#                 corrected a HTML error
#
# Version : 2.3 = corrected a HTML error
#                 moved table options to CSS file (as much as possible)
#                 cellspacing and cellpadding had to stay to keep layout OK
#
# Version : 2.4 = added code to overcome size problem when
#                 dfPerCentKBytesCapacity > 100
#                 added link to FilerView on Filer (http://filer/na_admin)
#                 added extra CSS stuff
#
# Version : 2.5 = correct error $diskReconstructingParityCount in HTML code
#                 correct timestamp
#                 made last updated info universal readable
#
# Version : 2.6 = Added config params to allow to control output
#                 This will make script also runable under a webserver and
#                 allow to create HTML on the fly
#
# Version : 2.7 = commented out Start en ending logging
#
# Version : 2.8 = added domain
#
# Version : 2.9 = added logic to correct incorrect kbytes
#
# Version : 3.0 = correct some thing and some HTML errors
#
# Version : 4.0 = changed the way to calc warnings & danger
#
# Version : 4.1 = changed to meet new battery info
#
# Version : 4.2 = adjusted calc warn & danger
#
# Version : 4.3 = correct precent calc when dfUsed = 0
#
# Version : 4.4 = replace http://filer with https://filer
#
# Version : 4.5 = capture snapshot space when they are reprensetend as .. instead of .snapshot
#
# Version : 5.0 = add snapshot, snapmirror and snapvault info
#
# Version : 5.1 = no longer reading filer info from .cfg but from Filers.txt
#
# Version : 5.2 = added several fill-ups if some elements were not discoverd
#
################################################################################

# compiler instructions
use strict;
use warnings;

####################################
# Modules                          #
####################################
#use IO::Handle;
#use Cwd;
use File::Basename;
use Getopt::Std;
use SNMP_util;
use Time::Local;
use Math::BigInt;
use Net::SMTP;
use Date::Calc qw(Add_Delta_Days Delta_Days Delta_DHMS);

#===========================================================================================================================================
# Definitions
#===========================================================================================================================================

# Constants
#   general purpose
use constant DEBUG                              => 0;                               # 0 = no debug info   1 = debug info

#   OID's we require to get volume metrics info
use constant OID_dfVolume_Index           		=> "1.3.6.1.4.1.789.1.5.6.0";       # number of df entries  (returns a number)  eg: 70
use constant OID_dfVolume_Indexes         		=> "1.3.6.1.4.1.789.1.5.4.1.1";     # list of indexes for volumes  => completed with .1 to .value of OID_dfVolume_Index  eg: 1, 2, 3, 4, ...
use constant OID_dfFileSys                		=> "1.3.6.1.4.1.789.1.5.4.1.2";     #  dfFileSys                    => completed with .value of OID_dfVolume_Indexes  (returns string) eg:  /vol/bvfwfs1/, /vol/bvfwfs1/.snapshot
use constant OID_dfPerCentKBytesCapacity  		=> "1.3.6.1.4.1.789.1.5.4.1.6";     #  dfPerCentKBytesCapacity      => completed with .value of OID_dfVolume_Indexes  (returns number)
use constant OID_dfHighTotalKBytes        		=> "1.3.6.1.4.1.789.1.5.4.1.14";    #  dfHighTotalKBytes            => completed with .value of OID_dfVolume_Indexes  (returns number)
use constant OID_dfLowTotalKBytes         		=> "1.3.6.1.4.1.789.1.5.4.1.15";    #  dfLowTotalKBytes             => completed with .value of OID_dfVolume_Indexes  (returns number)
use constant OID_dfHighUsedKBytes         		=> "1.3.6.1.4.1.789.1.5.4.1.16";    #  dfHighUsedKBytes             => completed with .value of OID_dfVolume_Indexes  (returns number)
use constant OID_dfLowUsedKBytes          		=> "1.3.6.1.4.1.789.1.5.4.1.17";    #  dfLowUsedKBytes              => completed with .value of OID_dfVolume_Indexes  (returns number)
use constant OID_dfHighAvailKBytes        		=> "1.3.6.1.4.1.789.1.5.4.1.18";    #  dfHighAvailKBytes            => completed with .value of OID_dfVolume_Indexes  (returns number)
use constant OID_dfLowAvailKBytes         		=> "1.3.6.1.4.1.789.1.5.4.1.19";    #  dfLowAvailKBytes             => completed with .value of OID_dfVolume_Indexes  (returns number)

# OID's we require to get vFiler info
use constant OID_vFiler_Index             		=> "1.3.6.1.4.1.789.1.16.2.0";      # number of indexes for vFilers  (returns a number)  eg: 4
use constant OID_vFiler_Indexes           		=> "1.3.6.1.4.1.789.1.16.3.1.1";    # list of indexes of vFilers   => completed with .1 to .value of OID_vFiler_Index (returns numbers)  eg:  1, 2, 3 , 4
use constant OID_vFilers_Names            		=> "1.3.6.1.4.1.789.1.16.3.1.2";    # list of names of vFilers     => completed with .value of OID_vFiler_Indexes (returns strings) eg: vfiler0, bvfdb1
use constant OID_Volume_Index             		=> "1.3.6.1.4.1.789.1.16.3.1.5";    # list of number of indexes per vfiler    => completed with .value of OID_vFiler_Indexes (returns numbers) eg: 1, 25, 3, 1
use constant OID_Volume_Indexes           		=> "1.3.6.1.4.1.789.1.16.5.1.2";    # list of indexes for volumes per vFiler  => completed with .value of OID_vFiler_Indexes and .value of OID_Volume_Index (returns numbers) eg: ....
use constant OID_Volume_Names             		=> "1.3.6.1.4.1.789.1.16.5.1.3";    # list of names of volumes per vFiler     => completed with .value of OID_vFiler_Indexes and .value of OID_Volume_Indexes (returns strings) eg:  /vol/bvfdb1, /vol/dmregie_d

use constant OID_productVersion 		  		=> "1.3.6.1.4.1.789.1.1.2.0";
use constant OID_productID 						=> "1.3.6.1.4.1.789.1.1.3.0";
use constant OID_productModel 					=> "1.3.6.1.4.1.789.1.1.5.0";
use constant OID_productSerialNum 				=> "1.3.6.1.4.1.789.1.1.9.0";
use constant OID_productFirmwareVersion 		=> "1.3.6.1.4.1.789.1.1.6.0";
use constant OID_cpuUpTime 						=> "1.3.6.1.4.1.789.1.2.1.1.0";

# environmental info - env
use constant OID_envFailedFanCount 				=> "1.3.6.1.4.1.789.1.2.4.2.0";
use constant OID_envFailedFanMessage 			=> "1.3.6.1.4.1.789.1.2.4.3.0";
use constant OID_envFailedPowerSupplyCount 		=> "1.3.6.1.4.1.789.1.2.4.4.0";
use constant OID_envFailedPowerSupplyMessage 	=> "1.3.6.1.4.1.789.1.2.4.5.0";

# disk info - disk
use constant OID_diskTotalCount 				=> "1.3.6.1.4.1.789.1.6.4.1.0";
use constant OID_diskActiveCount 				=> "1.3.6.1.4.1.789.1.6.4.2.0";
use constant OID_diskReconstructingCount 		=> "1.3.6.1.4.1.789.1.6.4.3.0";
use constant OID_diskReconstructingParityCount 	=> "1.3.6.1.4.1.789.1.6.4.4.0";
use constant OID_diskVerifyingParityCount 		=> "1.3.6.1.4.1.789.1.6.4.5.0";
use constant OID_diskFailedCount 				=> "1.3.6.1.4.1.789.1.6.4.7.0";
use constant OID_diskSpareCount 				=> "1.3.6.1.4.1.789.1.6.4.8.0";
use constant OID_diskFailedMessage 				=> "1.3.6.1.4.1.789.1.6.4.10.0";

# snapshot info
use constant OID_SSmonth                        => "1.3.6.1.4.1.789.1.5.5.2.1.2";
use constant OID_SSday                          => "1.3.6.1.4.1.789.1.5.5.2.1.3";
use constant OID_SShour                         => "1.3.6.1.4.1.789.1.5.5.2.1.4";
use constant OID_SSminute                       => "1.3.6.1.4.1.789.1.5.5.2.1.5";
use constant OID_SSname                         => "1.3.6.1.4.1.789.1.5.5.2.1.6";
use constant OID_SSvolname                      => "1.3.6.1.4.1.789.1.5.5.2.1.9";

# snapmirror info
use constant OID_SMsrc                          => "1.3.6.1.4.1.789.1.9.20.1.2";
use constant OID_SMdst                          => "1.3.6.1.4.1.789.1.9.20.1.3";
use constant OID_SMstatus                       => "1.3.6.1.4.1.789.1.9.20.1.4";
use constant OID_SMlag                          => "1.3.6.1.4.1.789.1.9.20.1.6";

# snapvault info
use constant OID_SVsrc                          => "1.3.6.1.4.1.789.1.19.11.1.2";
use constant OID_SVdst                          => "1.3.6.1.4.1.789.1.19.11.1.3";
use constant OID_SVstatus                       => "1.3.6.1.4.1.789.1.19.11.1.4";
use constant OID_SVlag                          => "1.3.6.1.4.1.789.1.19.11.1.6";

# other info
use constant OID_nvramBatteryStatus 			=> "1.3.6.1.4.1.789.1.2.5.1.0";
use constant OID_miscGlobalStatusMessage 		=> "1.3.6.1.4.1.789.1.2.2.25.0";

#===========================================================================================================================================
# Declararions of vars
#===========================================================================================================================================

# programwide variables

my $VERSION  = "5.2";                               # version
my $AUTHOR   = "Marc Engrie";                       # author

# some general vars
my ($fname, $dir, $ext) = fileparse($0, qr/\.[^.]*/);
my $logfile = sprintf("%s.log", $fname);
my ($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = localtime;
my $now;
my $LOG = 0;                                        # activate logging = 1  or not = 0 (default)

# for email
my $from    = $fname . "." . $ENV{COMPUTERNAME};
my $subject;
my $body;

# some overall used vars
my $temp;
my ($oid, $desc);
my ($oid1, $oid2);
my ($key, %value);
my $index;

# vars for config info
my $ConfigFile;
my %Config;                                         # hash containing all config info
my $FilerKey;
my $FilerValue;
my $CommunityKey;
my $CommunityValue;
my $HTMLKey;
my $HTMLValue;

# vars we require for volume metrics
my $dfFileSys;
my $dfPerCentKBytesCapacity;
my ($dfHighTotalKBytes, $dfLowTotalKBytes);
my ($dfHighUsedKBytes , $dfLowUsedKBytes);
my ($dfHighAvailKBytes, $dfLowAvailKBytes);

my ($dfName, $dfType, $dfPercent, $dfTotal, $dfUsed, $dfAvail, $dfDiff);
my ($df_switch, $df_status);

# vars we require to vFiler info
my @vFilers;
my @vFvols_Indexes;
my $vFvols_Index;
my @vFvols;

# vars we require for snapshot info
my (@arSSmonth, @arSSday, @arSShour, @arSSminute, @arSSname, @arSSvolname);
my ($NWyyyy, $NWmm);
my ($SSyyyy, $SSmm);

# vars we require for snapmirror info
my (@arSMsrc, @arSMdst, @arSMstatus, @arSMlag);
my @SMstatus = ('idle', 'transferring', 'pending', 'aborting', 'migrating', 'quiescing', 'resyncing', 'waiting');

# vars we require for snapvault info
my (@arSVsrc, @arSVdst, @arSVstatus, @arSVlag);
my @SVstatus = ('idle', 'transferring', 'pending', 'aborting', 'unknown',   'quiescing', 'resyncing');

# init hash of hashes - will contain all info related
my %Info =   ( "volume"  => { vfiler   => "vfiler0",             fullname => "name" ,      Type    => "X",    Percent => 0, Total => 0, Used => 0, Avail => 0 });
my %SSInfo = ( "000.000" => { datetime => "0000-00-00 hh:mm:ss", snapname => "SSSSSSSSSS", volname => "VVVVVVVVVV" });
my %SMInfo = ( "000.000" => { src      => "SSSSSSSSSS",          dst      => "DDDDDDDDDD", status  => 'idle', lag     => 0 });
my %SVInfo = ( "000.000" => { src     => "SSSSSSSSSS",           dst      => "DDDDDDDDDD", status  => 'idle', lag     => 0 });
my %MiscInfo;
my %vfInfo;
my %Alerts;

#===========================================================================================================================================
# Declararions of subroutines and functions
#===========================================================================================================================================

sub GetInfoFromFiler   ($$);
sub Create_HTML        ($);
sub List_vFilerInfo    ($$);
sub List_Volume        ($$$$);
sub CalcWarnDanger     ($);
sub List_SnapshotInfo  ($);
sub List_SnapmirrorInfo($);
sub List_SnapvaultInfo ($);
sub SendMail           ($$);
sub LoadAlerts		   ();
sub SaveAlerts         ();

#===========================================================================================================================================
# Main Code
#===========================================================================================================================================

# dump error message to error file
$SIG{__WARN__} = sub
{
    my $msg = shift;
    ($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = localtime();
    $now = sprintf("%04d-%02d-%02d %02d:%02d:%02d", 1900+$year, $mon+1, $day, $hour, $min, $sec);
    open  STDERR, ">>$fname.error.log";
    print STDERR  "$now -- ERROR: $msg\n";
    close STDERR;
};

####################################
# Get the parameters from the user
####################################

#get arguments
if (DEBUG)
{
  foreach $temp (0 .. $#ARGV)
  {
     #print STDERR "ARGV($temp): $ARGV[$temp]\n";
  }
}

# flush output
$| = 1;

# Variables setup
# set-up config file
if ( $#ARGV < 0 )
{
  	$ConfigFile = $fname . ".cfg";
}
elsif ( $ARGV[0] eq "-?" )
{
  	print STDERR "Usage: $fname [config]\n";
  	print STDERR "       config(optional) = name of the configuration file to be used\n";
  	print STDERR "                          (default: $fname.cfg)\n";
  	exit 255;
}
else
{
  	$ConfigFile = shift;
}

# read config file
open(CONFIG_FILE, "<$ConfigFile") or die "No conf file: $ConfigFile $!";
while (<CONFIG_FILE>)
{
  	next if /^\#/;
  	chomp;
  	next unless $_; # skip empty lines
  	s/\=/___EQ___/; # replace first equal sign
  	my ($key, $value) = split /___EQ___/;
  	$key =~ s/[\t ]//g;
  	# trim leading & trailing spaces
  	$value = join " ", grep { $_ } split /[\t ]/, $value;
  	$Config{$key} = $value;
  	#print STDERR "$key = $Config{$key}\n" if DEBUG;
}
close CONFIG_FILE;

# convert to uppercase
$Config{'OUTPUT'} =~ tr/a-z/A-Z/;

# set logging
if ( $Config{'LOGGING'} > 0 && $Config{'OUTPUT'} eq 'FILE' )
{
  	$LOG = 1;
}

# set-up timestamp
($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = localtime();
$now = sprintf("%04d-%02d-%02d %02d:%02d:%02d", 1900+$year, $mon+1, $day, $hour, $min, $sec);
print STDOUT "$now --  Starting $0 using config file: $ConfigFile\n" if $LOG;

#print STDERR "LOGGING = $Config{'LOGGING'}\n"         if DEBUG;
#print STDERR "OUTPUT = $Config{'OUTPUT'}\n"           if DEBUG;
#print STDERR "PATH_IMAGES = $Config{'PATH_IMAGES'}\n" if DEBUG;
#print STDERR "CSS = $Config{'CSS'}\n"                 if DEBUG;
#print STDERR "WARNING = $Config{'WARNING'}\n"         if DEBUG;
#print STDERR "WARNINGINC = $Config{'WARNINGINC'}\n"   if DEBUG;
#print STDERR "DANGER = $Config{'DANGER'}\n"           if DEBUG;
#print STDERR "DANGERINC = $Config{'DANGERINC'}\n"     if DEBUG;
#print STDERR "DOMAIN = $Config{'DOMAIN'}\n"           if DEBUG;
#print STDERR "SMTPSERVER = $Config{'SMTPSERVER'}\n"   if DEBUG;
#print STDERR "SSLAG = $Config{'SSLAG'}\n"             if DEBUG;
#print STDERR "SMLAG = $Config{'SMLAG'}\n"             if DEBUG;
#print STDERR "SVLAG = $Config{'SVLAG'}\n"             if DEBUG;

# set-up from emailadress
if($Config{'TO'} =~ m/(.*)@(.*)/)
{
	$from = $from . "@" . $2;
}
else
{
	undef $from;
}

if (DEBUG)
{
	($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = localtime();
	$now = sprintf("%04d-%02d-%02d %02d:%02d:%02d", 1900+$year, $mon+1, $day, $hour, $min, $sec);
	$subject = "Test email";
	$body    = $now . "\n" . $subject;
	SendMail $subject, $body;
}

LoadAlerts();

open  FILERS, ">Filers.txt";
open  VFILERS, ">vFilers.txt";

# open reference file
open TXT, "<$Config{'LISTFILERS'}" or die $!;
# read file
while (<TXT>)
{
    chomp;
    $FilerValue = $_;
    # remove leading & trailing spaces
    $FilerValue =~ s/\s+$//;
    $FilerValue =~ s/^\s+//;
     # to uppercase
    $FilerValue =~ tr/a-z/A-Z/;
    #printf "$filer\n";

    if (length($FilerValue) > 0)
    {

      	$| = 1;

      	$CommunityValue = 'public';

      	$HTMLValue = $Config{'HTMLROOT'} . $FilerValue . "_Status.html";

      	# set-up timestamp
    	($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = localtime();
    	$now = sprintf("%04d-%02d-%02d %02d:%02d:%02d", 1900+$year, $mon+1, $day, $hour, $min, $sec);
      	print STDOUT "$now --   Creating HTML file: $HTMLValue for filer: $FilerValue using community: $CommunityValue\n" if $LOG;
      	print STDERR "$now --   Creating HTML file: $HTMLValue for filer: $FilerValue using community: $CommunityValue\n" if DEBUG;

    	GetInfoFromFiler($FilerValue, $CommunityValue);
    	if ($Config{'OUTPUT'} eq 'FILE')
    	{
        	open(HTML_FILE, ">$HTMLValue") or die "Problem creating file: $HTMLValue $!";
        	select(HTML_FILE);
      	}
      	Create_HTML ($FilerValue);
    	if ( $Config{'OUTPUT'} eq 'FILE' )
      	{
        	# close file
        	close HTML_FILE;
      	}

      	# set-up timestamp
    	($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = localtime();
    	$now = sprintf("%04d-%02d-%02d %02d:%02d:%02d", 1900+$year, $mon+1, $day, $hour, $min, $sec);
      	print STDOUT "$now --   Completed HTML file creation for filer: $FilerValue \n" if $LOG;
      	print STDERR "$now --   Completed HTML file creation for filer: $FilerValue \n" if DEBUG;
    }
}

close VFILERS;
close FILERS;
close TXT;

SaveAlerts ();

# set-up timestamp
($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = localtime();
$now = sprintf("%04d-%02d-%02d %02d:%02d:%02d", 1900+$year, $mon+1, $day, $hour, $min, $sec);
print STDOUT "$now --  Ending $0 using config file: $ConfigFile\n" if $LOG;

exit 0;

#===========================================================================================================================================
# Subroutines
#===========================================================================================================================================

sub GetInfoFromFiler($$)
{

    my $filer = shift;
    my $community = shift;
	my $index;
	my @indexes;
	my $tmp;

	# clear hashes
	if (@vFilers) { @vFilers  = ()};
	if (%Info)    { %Info     = ()};
	if (%MiscInfo){ %MiscInfo = ()};

	if (%SSInfo)  { %SSInfo   = ()};
	if (%SMInfo)  { %SMInfo   = ()};
	if (%SVInfo)  { %SVInfo   = ()};

	# first load all dfVolumes and related metrics into hash, default vfiler = $filer
	#   get number of volumes
	#$index  = snmpget($community . '@' . $filer, OID_dfVolume_Index);

	#   get list of indexes for volumes
	@indexes    = &snmpwalk($community . '@' . $filer, OID_dfVolume_Indexes);
	foreach $desc (@indexes)
	{
		#printf "$desc\n" if DEBUG;
	    # split of oid and index
	    ($oid, $desc)  = split(':', $desc, 2);
	}
	#print STDERR "@indexes\n" if DEBUG;

	#   now load volumes and related metrics
	foreach $index (@indexes)
	{
		$dfFileSys = snmpget($community . '@' . $filer, OID_dfFileSys . "." . $index);
		$dfName = $dfFileSys;
		# remove trailing /
		$dfName =~ s/\/$//;
		# remove /vol/ if exists otherwise if is an aggregate
		if($dfName =~ m/\/vol\/*./i)
		{
			$dfName =~ s/\/vol\///i;
			$dfType = "V";
		}
		else
		{
			$dfType = "A"
		}
		if ($dfName =~ s/\/\.snapshot//i)
		{
			$dfName = $dfName . "_snap";
		}
		if ($dfName =~ s/\/\.\.//i)
		{
			$dfName = $dfName . "_snap";
		}
		#printf STDERR ("dfName  : %s\n", $dfName) if DEBUG;
		$dfPerCentKBytesCapacity = snmpget($community . '@' . $filer, OID_dfPerCentKBytesCapacity . "." . $index);
		$dfPerCentKBytesCapacity = unpack("I", pack("i", $dfPerCentKBytesCapacity));
		$dfHighTotalKBytes       = snmpget($community . '@' . $filer, OID_dfHighTotalKBytes . "." . $index);
		$dfHighTotalKBytes       = unpack("I", pack("i", $dfHighTotalKBytes));
		$dfLowTotalKBytes        = snmpget($community . '@' . $filer, OID_dfLowTotalKBytes. "." . $index);
		$dfLowTotalKBytes        = unpack("I", pack("i", $dfLowTotalKBytes));
		$dfHighUsedKBytes        = snmpget($community . '@' . $filer, OID_dfHighUsedKBytes . "." . $index);
		$dfHighUsedKBytes        = unpack("I", pack("i", $dfHighUsedKBytes));
		$dfLowUsedKBytes         = snmpget($community . '@' . $filer, OID_dfLowUsedKBytes . "." . $index);
		$dfLowUsedKBytes         = unpack("I", pack("i", $dfLowUsedKBytes));
		$dfHighAvailKBytes       = snmpget($community . '@' . $filer, OID_dfHighAvailKBytes . "." . $index);
		$dfHighAvailKBytes       = unpack("I", pack("i", $dfHighAvailKBytes));
		$dfLowAvailKBytes        = snmpget($community . '@' . $filer, OID_dfLowAvailKBytes . "." . $index);
		$dfLowAvailKBytes        = unpack("I", pack("i", $dfLowAvailKBytes));

	    # calculate actual kBytes
	    # Round the numbers, from kilobytes to gigabytes
	    $dfTotal = Math::BigInt->new(2**32);
	    $dfTotal->bmul($dfHighTotalKBytes);
	    $dfTotal->badd($dfLowTotalKBytes);
	    $dfTotal->bdiv(1048576);
	    #printf STDERR ("dfTotal : %s\n", $dfTotal) if DEBUG;

	    $dfUsed = Math::BigInt->new(2**32);
	    $dfUsed->bmul($dfHighUsedKBytes);
	    $dfUsed->badd($dfLowUsedKBytes);
	    $dfUsed->bdiv(1048576);
	    #printf STDERR ("dfUsed  : %s\n", $dfUsed) if DEBUG;

	    $dfAvail = Math::BigInt->new(2**32);
	    $dfAvail->bmul($dfHighAvailKBytes);
	    $dfAvail->badd($dfLowAvailKBytes);
	    $dfAvail->bdiv(1048576);
	    #printf STDERR ("dfAvail : %s\n", $dfAvail) if DEBUG;

	    # check for wrong numbers
	    if ( $dfUsed > $dfTotal )
	    {
	      	$dfUsed = 0;
	    }

	    if ( $dfAvail > $dfTotal )
	    {
	      	$dfUsed = 0;
	    }

	    if ( ($dfAvail + $dfUsed) > $dfTotal )
	    {
	      	$dfUsed = 0;
	      	$dfAvail = 0;
	    }

	    # correct for problem when snapshotspace > 100%
	    # allow margin of 3 GB
	    $dfDiff = abs($dfTotal - $dfUsed - $dfAvail);
	    if ($dfDiff > 3)
	    {
	      	if ( $dfPerCentKBytesCapacity > 100 )
	      	{
	        	$dfAvail = $dfTotal - $dfUsed;
	      	}
	    }

	    # cut excess
	    $dfPercent = $dfPerCentKBytesCapacity > 100 ? 100 : $dfPerCentKBytesCapacity;

		if ($dfUsed == 0)
		{
			$dfPercent = 0;
		}

		# add all to hash
		$Info{$dfName}{"vfiler"}   = $filer;
		$Info{$dfName}{"fullname"} = $dfFileSys;
		$Info{$dfName}{"Type"}     = $dfType;
		$Info{$dfName}{"Percent"}  = $dfPercent;
		$Info{$dfName}{"Total"}    = $dfTotal;
		$Info{$dfName}{"Used"}     = $dfUsed;
		$Info{$dfName}{"Avail"}    = $dfAvail;
	}

	# next load all vfilers in array
	#   get number of vfilers
	$index = snmpget($community . '@' . $filer, OID_vFiler_Index);
	#print STDERR "$index\n" if DEBUG;
	if($index > 0)
	{
		#   get list of indexes for vfilers
		@indexes = &snmpwalk($community . '@' . $filer, OID_vFiler_Indexes);
		foreach $desc (@indexes)
		{
			#printf "$desc\n" if DEBUG;
		    # split of oid and index
		    ($oid, $desc)  = split(':', $desc, 2);
		}
		#print STDERR "@indexes\n" if DEBUG;

		#   now load names
		# first create 0-element of array as index always starts from 1
		push(@vFilers, $filer);
        printf FILERS "$filer\n";
		foreach $index (@indexes)
		{
			$temp = snmpget($community . '@' . $filer, OID_vFilers_Names . "." . $index);
			push(@vFilers, $temp);
			# skip vfiler0
       	    if ( $temp !~ m/vfiler0/i )
            {
                printf VFILERS "$temp\n";
            }
		}

		#   now load volumes
		foreach $index (@indexes)
		{

			# next load all vfilers in array
			#   get number of vfilers
			#$index  = snmpget($community . '@' . $filer, OID_vFiler_Index);

			#   get list of indexes for vfilers
			@vFvols_Indexes = &snmpwalk($community . '@' . $filer, OID_Volume_Indexes. "." . $index);
			foreach $desc (@vFvols_Indexes)
			{
				#printf "$desc\n" if DEBUG;
			    # split of oid and index
			    ($oid, $desc)  = split(':', $desc, 2);
			}
			#print STDERR "@vFvols_Indexes\n" if DEBUG;

			#   now load names
			if (DEBUG)
			{
				foreach $vFvols_Index (@vFvols_Indexes)
				{
					$temp = snmpget($community . '@' . $filer, OID_Volume_Names . "." . $index . "." . $vFvols_Index);
					printf STDERR "vFiler volume: %02d.%02d -> %s\n", $index, $vFvols_Index, $temp;
				}
			 }

			# then find volumes on each vfiler, and update hash with vfiler-name
			foreach $vFvols_Index (@vFvols_Indexes)
			{
				$temp = snmpget($community . '@' . $filer, OID_Volume_Names . "." . $index . "." . $vFvols_Index);

				# skip .1.1 = root = /
				unless($index == 1 && $vFvols_Index == 1)
				{
					# remove trailing /
					$temp =~ s/\/$//i;
					# remove /vol/
					$temp =~ s/\/vol\///i;
					# change %Info => volume => vfiler to this vfiler
					if(defined $Info{$temp})
					{
						$Info{$temp}{"vfiler"} = $vFilers[$index];
						# also for snapshot volume info
						$temp = $temp . "_snap";
						if(defined $Info{$temp})
						{
							$Info{$temp}{"vfiler"} = $vFilers[$index];
						}
					}
				}
				#printf STDERR "vFiler vol1: %s  - vFiler vol2: %s\n", $temp, $Info{$temp}{"vfiler"} if DEBUG;
			}
		}
	}

	if(DEBUG)
	{
	    for $key ( sort keys %Info )
	    {
	        printf STDERR "key: %-25s =>  vfil:%-10s - ful:%-35s - typ: %1s - tot:%5sGB  - usd:%5sGB  - avl:%5sGB - per:%3s \n", $key, $Info{$key}{"vfiler"}, $Info{$key}{"fullname"}, $Info{$key}{"Type"}, $Info{$key}{"Total"}, $Info{$key}{"Used"}, $Info{$key}{"Avail"}, $Info{$key}{"Percent"};
	    }
	}

  	# Get all of our interesting status details.
  	# productVersion
  	$temp = snmpget($community . '@' . $filer, OID_productVersion);
  	chomp $temp;
	($temp, $tmp) = split(/:/, $temp);
  	$temp =~ s/"//g; #" -> to fool editor
  	#printf STDERR ("productVersion: %s\n", $temp) if DEBUG;
  	$MiscInfo{"productVersion"} = $temp;

	# productID
	$temp = snmpget($community . '@' . $filer, OID_productID);
	chomp $temp;
	$temp =~ s/"//g; #" -> to fool editor
	#printf STDERR ("productId: %s\n", $temp) if DEBUG;
	$MiscInfo{"productID"} = $temp;

	$temp = snmpget($community . '@' . $filer, OID_productModel);
	chomp $temp;
	$temp =~ s/"//g; #" -> to fool editor
	# printf STDERR ("productModel: %s\n", $temp) if DEBUG;
	$MiscInfo{"productModel"} = $temp;

	# productSerialNum
	$temp = snmpget($community . '@' . $filer, OID_productSerialNum);
	chomp $temp;
	$temp =~ s/"//g;
	#printf STDERR ("productSerialNum: %s\n", $temp) if DEBUG;
	$MiscInfo{"productSerialNum"} = $temp;

	# productFirmwareVersion
	$temp = snmpget($community . '@' . $filer, OID_productFirmwareVersion);
	chomp $temp;
	$temp =~ s/"//g; #" -> to fool editor
	#printf STDERR ("productFirmwareVersion: %s\n", $temp) if DEBUG;
	$MiscInfo{"productFirmwareVersion"} = $temp;

	# cpuUpTime
	$temp = snmpget($community . '@' . $filer, OID_cpuUpTime);
	chomp $temp;
	if (index($temp,',') > 1)
	{
    	($day, $temp) = split(/,/ , $temp);
    }
    else
    {
    	($day, $temp) = split(/ / , $temp, 2);
    }
	$day =~ s/^\s+//;
	$day =~ s/\s+$//;
	#printf STDERR ("cpuUpDays: %s\n", $day) if DEBUG;
	$MiscInfo{"cpuUpDays"} = $day;
	if($temp)
	{
    	($hour, $min, $sec) = split(/:/, $temp);
    	$hour =~ s/^\s+//;
    	$hour =~ s/\s+$//;
    	$min =~ s/^\s+//;
    	$min =~ s/\s+$//;
    	if(index($sec,'.') > 1)
    	{
            ($sec, $desc) = split(/\./, $sec);
        }
        else
        {
            ($sec, $desc) = split(/ /, $sec);
        }
    	$sec =~ s/^\s+//;
    	$sec =~ s/\s+$//;
    	#printf STDERR ("cpuUpHours: %s\n", $hour) if DEBUG;
    	#printf STDERR ("cpuUpMins: %s\n", $min) if DEBUG;
    	#printf STDERR ("cpuUpSecs: %s\n", $sec) if DEBUG;
	}
    else
    {
        $hour = 0;
        $min  = 0;
        $sec  = 0;
    }
	$MiscInfo{"cpuUpHours"} =  sprintf("%02d", $hour);
	$MiscInfo{"cpuUpMins"}  =  sprintf("%02d", $min);
	$MiscInfo{"cpuUpSecs"}  =  sprintf("%02d", $sec);

	# envFailedFanCount
	$temp = snmpget($community . '@' . $filer, OID_envFailedFanCount);
	chomp $temp;
	#printf STDERR ("envFailedFanCount: %s\n", $temp) if DEBUG;
	$MiscInfo{"envFailedFanCount"} = $temp;
	$MiscInfo{"FailedFan"} = $temp > 0 ? "NOK" : "OK";

	# envFailedFanMessage
	$temp = snmpget($community . '@' . $filer, OID_envFailedFanMessage);
	chomp $temp;
	$temp =~ s/"//g; #" -> to fool editor
	#printf STDERR ("envFailedFanMessage: %s\n", $temp) if DEBUG;
	$MiscInfo{"envFailedFanMessage"} = $temp;

	if ($MiscInfo{"FailedFan"} eq "NOK" )
	{
		$subject = "Failed Fan on $filer";
		$body    = $MiscInfo{"envFailedFanMessage"};
		if ( $Alerts{$filer}{Fans} < 3 )
		{
			if (SendMail($subject, $body))
			{
				$Alerts{$filer}{Fans}++;
			}
		}
	}
	else
	{
		$Alerts{$filer}{Fans} = 0;
	}

	# envFailedPowerSupplyCount
	$temp = snmpget($community . '@' . $filer, OID_envFailedPowerSupplyCount);
	chomp $temp;
	#printf STDERR ("envFailedPowerSupplyCount: %s\n", $temp) if DEBUG;
	$MiscInfo{"envFailedPowerSupplyCount"} = $temp;
	$MiscInfo{"FailedPowerSupply"} = $temp > 0 ? "NOK" : "OK";

	# envFailedPowerSupplyMessage
	$temp = snmpget($community . '@' . $filer, OID_envFailedPowerSupplyMessage);
	chomp $temp;
	$temp =~ s/"//g; #" -> to fool editor
	#printf STDERR ("envFailedPowerSupplyMessage: %s\n", $temp) if DEBUG;
	$MiscInfo{"envFailedPowerSupplyMessage"} = $temp;

	if ($MiscInfo{"FailedPowerSupply"} eq "NOK" )
	{
		$subject = "Failed Power Supply on $filer";
		$body    = $MiscInfo{"envFailedPowerSupplyMessage"};
		if ( $Alerts{$filer}{PowerSupply} < 3 )
		{
			if (SendMail($subject, $body))
			{
				$Alerts{$filer}{PowerSupply}++;
			}
		}
	}
	else
	{
		$Alerts{$filer}{PowerSupply} = 0;
	}

	# diskTotalCount
	$temp = snmpget($community . '@' . $filer, OID_diskTotalCount);
	chomp $temp;
	#printf STDERR ("diskTotalCount: %s\n", $temp) if DEBUG;
	$MiscInfo{"diskTotalCount"} = $temp;

	# diskActiveCount
	$temp = snmpget($community . '@' . $filer, OID_diskActiveCount);
	chomp $temp;
	#printf STDERR ("diskActiveCount: %s\n", $temp) if DEBUG;
	$MiscInfo{"diskActiveCount"} = $temp;

	# diskReconstructingCount
	$temp = snmpget($community . '@' . $filer, OID_diskReconstructingCount);
	chomp $temp;
	#printf STDERR ("diskReconstructingCount: %s\n", $temp) if DEBUG;
	$MiscInfo{"diskReconstructingCount"} = $temp;
	$MiscInfo{"diskReconstructing"} = $temp > 0 ? "NOK" : "OK";

	# diskReconstructingParityCount
	$temp = snmpget($community . '@' . $filer, OID_diskReconstructingParityCount);
	chomp $temp;
	#printf STDERR ("diskReconstructingParityCount: %s\n", $temp) if DEBUG;
	$MiscInfo{"diskReconstructingParityCount"} = $temp;
	$MiscInfo{"diskReconstructingParity"} = $temp > 0 ? "NOK" : "OK";

	# diskVerifyingParityCount
	$temp = snmpget($community . '@' . $filer, OID_diskVerifyingParityCount);
	chomp $temp;
	#printf STDERR ("diskVerifyingParityCount: %s\n", $temp) if DEBUG;

	# diskFailedCount
	$temp = snmpget($community . '@' . $filer, OID_diskFailedCount);
	chomp $temp;
	#printf STDERR ("diskFailedCount: %s\n", $temp) if DEBUG;
	$MiscInfo{"diskFailedCount"} = $temp;
	$MiscInfo{"diskFailed"} = $temp > 0 ? "NOK" : "OK";

	# diskSpareCount
	$temp = snmpget($community . '@' . $filer, OID_diskSpareCount);
	chomp $temp;
	#printf STDERR ("diskSpareCount: %s\n", $temp) if DEBUG;
	$MiscInfo{"diskSpareCount"} = $temp;

	# diskFailedMessage
	$temp = snmpget($community . '@' . $filer, OID_diskFailedMessage);
	chomp $temp;
	$temp =~ s/"//g; #" -> to fool editor
	#printf STDERR ("diskFailedMessage: %s\n", $temp) if DEBUG;
	$MiscInfo{"diskFailedMessage"} = $temp;

	if ($MiscInfo{"diskFailed"} eq "NOK" )
	{
		$subject = "Failed Disk on $filer";
		$body    = $MiscInfo{"diskFailedMessage"};
		if ( $Alerts{$filer}{Disks} < 3 )
		{
			if (SendMail($subject, $body))
			{
				$Alerts{$filer}{Disks}++;
			}
		}
	}
	else
	{
		$Alerts{$filer}{Disks} = 0;
	}

	# nvramBatteryStatus
	#    1: ok
	#    2: partiallyDischarged
	#    3: fullyDisharged
	#    4: notPresent
	#    5: nearEndOfLife
	#    6: atEndOfLife
	#    7: unknown
  	#    8: overCharged
  	#    9: fullyCharged

	$temp = snmpget($community . '@' . $filer, OID_nvramBatteryStatus);
	chomp $temp;

	$MiscInfo{"BatteryStatus"} = $temp ne '1' ? "NOK" : "OK";

	if($temp eq '1')
	{
	  	$temp = "OK";
	}
	elsif($temp eq '2')
	{
	  	$temp = "Partially Discharged";
	}
	elsif($temp eq '3')
	{
	  	$temp = "Fully Discharged";
	}
	elsif($temp eq '4')
	{
	  	$temp = "Not Present";
	}
	elsif($temp eq '5')
	{
	  	$temp = "Near End-Of-Life";
	}
	elsif($temp eq '6')
	{
	  	$temp = "At End-Of-Life";
	}
	elsif($temp eq '7')
	{
	  	$temp = "Unknown";
	}
	elsif($temp eq '8')
	{
	  	$temp = "Overcharged";
	}
	elsif($temp eq '9')
	{
	  	$temp = "Fully Charged";
	  	$MiscInfo{"BatteryStatus"} = "OK";
	}
	else
	{
	    $temp = "Unknown";
	}
	#printf STDERR ("nvramBatteryStatus: %s\n", $temp) if DEBUG;
	$MiscInfo{"nvramBatteryStatus"} = $temp;

	if ($MiscInfo{"BatteryStatus"} eq "NOK" )
	{
		$subject = "Failed Battery on $filer";
		$body    = "Battery " . $temp;
		if ( $Alerts{$filer}{Battery} < 3 )
		{
			if (SendMail($subject, $body))
			{
				$Alerts{$filer}{Battery}++;
			}
		}
	}
	else
	{
		$Alerts{$filer}{Battery} = 0;
	}

	# miscGlobalStatusMessage
	$temp = snmpget($community . '@' . $filer, OID_miscGlobalStatusMessage);
	chomp $temp;
	$temp =~ s/"//g; #"  -> thisone is needed to correct problem with editor :-)
	$tmp = "NOK";
	if($temp =~ m/normal/)
	{
	  	$tmp = "OK";
	}
	#printf STDERR ("miscGlobalStatusMessage: %s\n", $tmp) if DEBUG;
	$MiscInfo{"GlobalStatus"} = $tmp;
	$MiscInfo{"miscGlobalStatusMessage"} = $temp;

	if ($MiscInfo{"GlobalStatus"} eq "NOK" )
	{
		$subject = "Global Status problem on $filer";
		$body    = $MiscInfo{"miscGlobalStatusMessage"};
		if ( $Alerts{$filer}{Global} < 3 )
		{
			if (SendMail($subject, $body))
			{
				$Alerts{$filer}{Global}++;
			}
		}
	}
	else
	{
		$Alerts{$filer}{Global} = 0;
	}

}

#===========================================================================================================================================

sub Create_HTML($)
{
    my $filer = shift;
    #<link href='$Config{"CSS"}' rel='stylesheet' type='text/css' />

	# <HEAD>
  	print <<HEADER;
<!DOCTYPE HTML PUBLIC '-//W3C//DTD HTML 4.01 Transitional//EN' 'http://www.w3.org/TR/html4/loose.dtd'>
<html>
  <head>
    <title>FilerStatus : $filer</title>
    <link href='$Config{"CSS"}' rel='stylesheet' type='text/css' />
    <meta http-equiv='Content-Language' content='en-gb' />
    <meta http-equiv='Content-Type' content='text/html; charset=ISO-8859-1' />
    <meta http-equiv='Cache-Control' content='no-cache' />
    <meta http-equiv='Pragma' content='no-cache' />
    <meta http-equiv='Refresh' content='300' />
</head>
HEADER

  # <BODY>
  # first the general stuf
  print <<BODYHEAD;
<body>
  <div class='Page'>
    <h1><a href='https://$filer.$Config{"DOMAIN"}/na_admin'target='_blank'>$filer</a></h1>
    <hr />
    This page was updated on: <b>$now</b>
    <hr />
    <br />
    <table class='tb_globalstatus' cellspacing='0'>
	  <tr>
		<td class='td_globalstatus_$MiscInfo{"GlobalStatus"}'><b>$MiscInfo{"miscGlobalStatusMessage"}</b></td>
	  </tr>
	</table>
	<table class='tb_status' cellpadding='0' cellspacing='0'>
	  <tr>
		<td class='td_OK'>
		  <table class='table_substatus' cellpadding='2' cellspacing='0'>
			<tr>
			  <td class='td_OK'>Filer Model:</td>
			  <td class='td_OK'>$MiscInfo{"productModel"}</td>
			</tr>
			<tr>
  			  <td class='td_OK'>Product ID:</td>
			  <td class='td_OK'>$MiscInfo{"productID"}</td>
			</tr>
			<tr>
			  <td class='td_OK'>Product SN:</td>
			  <td class='td_OK'>$MiscInfo{"productSerialNum"}</td>
			</tr>
			<tr>
			  <td class='td_OK'><br /></td>
			  <td class='td_OK'><br /></td>
			</tr>
			<tr>
			  <td class='td_$MiscInfo{"FailedFan"}'>Fans Failed:</td>
			  <td class='td_$MiscInfo{"FailedFan"}'>$MiscInfo{"envFailedFanCount"}</td>
			</tr>
			<tr>
			  <td class='td_$MiscInfo{"FailedFan"}'>Fans Failed:</td>
			  <td class='td_$MiscInfo{"FailedFan"}'>$MiscInfo{"envFailedFanMessage"}</td>
			</tr>
			<tr>
			  <td class='td_$MiscInfo{"FailedPowerSupply"}'>Power Supplies Failed:</td>
			  <td class='td_$MiscInfo{"FailedPowerSupply"}'>$MiscInfo{"envFailedPowerSupplyCount"}</td>
			</tr>
			<tr>
			  <td class='td_$MiscInfo{"FailedPowerSupply"}'>Power Supply Msg:</td>
			  <td class='td_$MiscInfo{"FailedPowerSupply"}'>$MiscInfo{"envFailedPowerSupplyMessage"}</td>
			</tr>
			<tr>
			  <td class='td_$MiscInfo{"BatteryStatus"}'>Battery Status:</td>
			  <td class='td_$MiscInfo{"BatteryStatus"}'>$MiscInfo{"nvramBatteryStatus"}</td>
			</tr>
			<tr>
			  <td class='td_OK'><br /></td>
			  <td class='td_OK'><br /></td>
			</tr>
			<tr>
			  <td class='td_OK'>System Uptime:</td>
			  <td class='td_OK'>$MiscInfo{"cpuUpDays"}, $MiscInfo{"cpuUpHours"}:$MiscInfo{"cpuUpMins"}:$MiscInfo{"cpuUpSecs"}</td>
			</tr>
		  </table>
		</td>
		<td class='td_OK'>
		  <table class='table_substatus' cellspacing='0'>
			<tr>
			  <td class='td_OK'>Firmware Version:</td>
			  <td class='td_OK'>$MiscInfo{"productFirmwareVersion"}</td>
			</tr>
			<tr>
			  <td class='td_OK'>Data Ontap Version:</td>
			  <td class='td_OK'>$MiscInfo{"productVersion"}</td>
			</tr>
			<tr>
			  <td class='td_OK'><br /></td>
			  <td class='td_OK'><br /></td>
			</tr>
			<tr>
			  <td class='td_OK'><br /></td>
			  <td class='td_OK'><br /></td>
			</tr>
			<tr>
			  <td class='td_OK'>Total Disks:</td>
			 <td class='td_OK'>$MiscInfo{"diskTotalCount"}</td>
			</tr>
			<tr>
			  <td class='td_OK'>Spare Disks:</td>
			  <td class='td_OK'>$MiscInfo{"diskSpareCount"}</td>
			</tr>
			<tr>
			  <td class='td_OK'>Active Disks:</td>
			  <td class='td_OK'>$MiscInfo{"diskActiveCount"}</td>
			</tr>
			<tr>
			  <td class='td_$MiscInfo{"diskReconstructing"}'>Rebuilding Disks:</td>
			  <td class='td_$MiscInfo{"diskReconstructing"}'>$MiscInfo{"diskReconstructingCount"}</td>
			</tr>
			<tr>
			  <td class='td_$MiscInfo{"diskReconstructingParity"}'>Rebuilding Parity Disks</td>
			  <td class='td_$MiscInfo{"diskReconstructingParity"}'>$MiscInfo{"diskReconstructingParityCount"}</td>
			</tr>
			<tr>
			  <td class='td_$MiscInfo{"diskFailed"}'>Failed Disks:</td>
			  <td class='td_$MiscInfo{"diskFailed"}'>$MiscInfo{"diskFailedCount"}</td>
			</tr>
			<tr>
			  <td class='td_$MiscInfo{"diskFailed"}'>Disk Message:</td>
			  <td class='td_$MiscInfo{"diskFailed"}'>$MiscInfo{"diskFailedMessage"}</td>
			</tr>
		  </table>
		</td>
	  </tr>
	</table>
	<br />
BODYHEAD

  # now the volumes
  # header of tables
  print <<BEGINVOLS;
	<table class='tb_filesys' cellspacing='1'>
	  <tr>
		<td class='tt_filesys' colspan='6'>vFiler & Volume Info</td>
	  </tr>
	  <tr>
		<td class='th_volumename'>Volume Name</td>
		<td class='th_usage'     >Usage</td>
		<td class='th_capacity'  >Capacity</td>
		<td class='th_used'      >Used</td>
		<td class='th_free'      >Free</td>
		<td class='th_precentage'>%</td>
	  </tr>
BEGINVOLS

	# first filer volumes
	#print STDERR "1: -> @vFilers\n" if DEBUG;

	List_vFilerInfo($filer, $filer);
	shift(@vFilers);
	#print STDERR "2: -> @vFilers\n" if DEBUG;

	if (@vFilers)
	{
		if (scalar(@vFilers) >0)
		{
			# next vfiler0 volumes -> vfile0 = based filer
			# List_vFilerInfo($filer ,"vfiler0");
			shift(@vFilers);
			#print STDERR "3: -> @vFilers\n" if DEBUG;

			# rest of vfilers volumes
			if (@vFilers)
			{
				if (scalar(@vFilers) > 0)
				{
					foreach $temp (sort @vFilers)
					{
						List_vFilerInfo($filer, $temp);
					}
				}
			}
		}
	}

  	# close table
  	print <<ENDVOLS;
	  <tr>
		<td class='tf_filesys' colspan='6'>&nbsp</td>
	  </tr>
	</table>
	<br />
ENDVOLS

  # now the snapshots
  # header of table
  print <<BEGINSNAPSHOTS;
	<table class='tb_snapshot' cellspacing='1'>
	  <tr>
		<td class='tt_snapshot' colspan='4' style="text-align: left">Snapshot Info</td>
	  </tr>
	  <tr>
		<td class='th_SSvolname'  >Volume Name</td>
		<td class='th_SSname'     >Snapshot Name</td>
		<td class='th_SSdatetaken'>Taken on</td>
		<td class='th_SSlag'      >Lag</td>
	  </tr>
BEGINSNAPSHOTS

    List_SnapshotInfo($filer);

  	# close table
  	print <<ENDSNAPSHOTS;
	  <tr>
		<td class='tf_snapshot' colspan='4'>&nbsp</td>
	  </tr>
	</table>
	<br />
ENDSNAPSHOTS

  # now the snapmirrors
  # header of table
  print <<BEGINSNAPMIRRORS;
	<table class='tb_snapmirror' cellspacing='1'>
	  <tr>
		<td class='tt_snapmirror' colspan='4' style="text-align: left">Snapmirror Info</td>
	  </tr>
	  <tr>
		<td class='th_SMsrc'   >Source</td>
		<td class='th_SMdst'   >Destination</td>
		<td class='th_SMstatus'>Status</td>
		<td class='th_SMlag'   >Lag</td>
	  </tr>
BEGINSNAPMIRRORS

    List_SnapmirrorInfo($filer);

  	# close table
  	print <<ENDSNAPMIRRORS;
	  <tr>
		<td class='tf_snapmirror' colspan='4'>&nbsp</td>
	  </tr>
	</table>
	<br />
ENDSNAPMIRRORS

  # now the snapvaults
  # header of table
  print <<BEGINSNAPVAULTS;
	<table class='tb_snapvault' cellspacing='1'>
	  <tr>
		<td class='tt_snapvault' colspan='4'>Snapvault Info</td>
	  </tr>
	  <tr>
		<td class='th_SVsrc'   >Source</td>
		<td class='th_SVdst'   >Destination</td>
		<td class='th_SVstatus'>Status</td>
		<td class='th_SVlag'   >Lag</td>
	  </tr>
BEGINSNAPVAULTS

    List_SnapvaultInfo($filer);

  	# close table
  	print <<ENDSNAPVAULTS;
	  <tr>
		<td class='tf_snapvault' colspan='4'>&nbsp</td>
	  </tr>
	</table>
	<br />
ENDSNAPVAULTS

  	# close HMTL page
  	print <<FOOTER;
	<hr />
	<h3>This page was created using <b>$fname</b> Version: $VERSION written by <b><a class='bluelink' href="http://marc.engrie.be/">$AUTHOR</a></b></h3>
  </div>
</body>
</html>
FOOTER

}

#===========================================================================================================================================

sub List_vFilerInfo($$)
{

	my $filer= shift;
	my $vFiler= shift;

    my $df_switch = 2;

	#printf STDERR ("vFiler -> %s\n", $vFiler) if DEBUG;

	# clear hash of vFiler
	if (%vfInfo) { %vfInfo = ()};
	# load info for this vFiler
	foreach $temp (sort keys %Info)
	{
		if($Info{$temp}{"vfiler"} eq $vFiler)
		{
			$vfInfo{$temp} = $Info{$temp};
		}
	}

	# list aggregates first if with list volumes of physical filer
	if($filer eq $vFiler)
	{
		foreach $temp (sort keys %vfInfo)
		{
			if($vfInfo{$temp}{"Type"} eq "A")
			{
				$df_switch = List_Volume($filer, $vFiler, $temp, $df_switch);
				# delete this entry
				delete $vfInfo{$temp};
			}
		}

		# list vol0 - always there
		$temp = "vol0";
		if($vfInfo{$temp})
		{
			$df_switch = List_Volume($filer, $vFiler, $temp, $df_switch);
			# delete this entry
			delete $vfInfo{$temp};
		}

		$temp = "vol0_snap";
		if($vfInfo{$temp})
		{
			$df_switch = List_Volume($filer, $vFiler, $temp, $df_switch);
			# delete this entry
			delete $vfInfo{$temp};
		}
	}

	# list vFiler info first if not physical filer
	if($filer ne $vFiler)
	{
		$temp = $vFiler;
		$df_switch = List_vFiler($filer, $vFiler, $temp, $df_switch);
		# delete this entry
		delete $vfInfo{$temp};
		$temp = $vFiler . "_snap";
		if ($vfInfo{$vFiler})
		{
			$df_switch = List_Volume($filer, $vFiler, $temp, $df_switch);
			# delete this entry
			delete $vfInfo{$temp};
		}
	}

	foreach $temp ( sort keys %vfInfo )
	{
		if($temp ne $vFiler)
		{
			#printf STDERR ("Info{temp}{vFiler} -> %-20s -> %-35s\n", $vfInfo{$temp}{"vfiler"}, $temp) if DEBUG;
			$df_switch = List_Volume($filer, $vFiler, $temp, $df_switch);
		}
	}
}

#==============================================================================================================================

sub List_vFiler($$$$)
{
	my $filer= shift;
	my $vFiler= shift;
	my $vol = shift;
	my $df_switch = shift;

  	my $df_status;
  	my $df_row;
  	my $gauge_width_GREEN;
  	my $gauge_width_YELLOW;
  	my $gauge_width_RED;
  	my $link;
  	my $warn;
  	my $danger;


	# set switches
	$df_switch = $df_switch == 1 ? 2 : 1;

	# cut excess
	$df_status = $vfInfo{$vol}{Percent} > 100 ? 100 : $vfInfo{$vol}{Percent};

	#define levels
	($warn, $danger) = CalcWarnDanger($vfInfo{$vol}{Total});


	# define colors etc.
	if ( $df_status <= $warn )
	{
	  	if(uc($vFiler) eq uc($vol))
		{
		  	$df_row = "td_row" . "_VFILER";
		}
		else
		{
			$df_row = "td_row" . $df_switch . "_GREEN";
		}
	  	$gauge_width_GREEN  = $df_status;
	  	$gauge_width_YELLOW = 0;
	  	$gauge_width_RED    = 0;;
	}
	elsif ( $df_status <= $danger )
	{
	  	$df_row = "td_row_YELLOW";
	  	$gauge_width_GREEN  = $warn;
	  	$gauge_width_YELLOW = $df_status - $warn;
	  	$gauge_width_RED    = 0;
	}
	else
	{
	  	$df_row = "td_row_RED";
	  	$gauge_width_GREEN  = $warn;
	  	$gauge_width_YELLOW = $danger - $warn;
	  	$gauge_width_RED    = $df_status - $danger;
	}

	$link = $filer;
	$link = $link . "_" . $vfInfo{$vol}{"fullname"};
	$link =~ tr/A-Z/a-z/;
	$link =~ s/vol//;
	$link =~ s/\///g;
	$link =~ s/\.snapshot//;
	$link =~ s/\.\.//;

	$temp = uc($vfInfo{$vol}{"vfiler"});
	# print table row
	if ( $gauge_width_RED > 0 )
	{
		print <<DF_ROW_RED;
	  <tr>
		<td class='$df_row'  align='left'><a style='color:white' href='$Config{"AREF"}$link&cfg=$filer.cfg'><b>$temp</b></a></td>
		<td class='td_row_GAUGE'>
		<img class='img_gauge' alt='$vfInfo{$vol}{"vfiler"}' src='$Config{"PATH_IMAGES"}/gauge_GREEN.jpg' width='$gauge_width_GREEN' /><img class='img_gauge' alt='$vfInfo{$vol}{"vfiler"}' src='$Config{"PATH_IMAGES"}/gauge_YELLOW.jpg' width='$gauge_width_YELLOW' /><img class='img_gauge' alt='$vfInfo{$vol}{"vfiler"}' src='$Config{"PATH_IMAGES"}/gauge_RED.jpg' width='$gauge_width_RED' />
DF_ROW_RED
	}
	elsif ( $gauge_width_YELLOW > 0 )
	{
 		print <<DF_ROW_YELLOW;
	  <tr>
		<td class='$df_row' align='left'><a style='color:white' href='$Config{"AREF"}$link&cfg=$filer.cfg'><b>$temp</b></a></td>
		<td class='td_row_GAUGE'>
		<img class='img_gauge' alt='$vfInfo{$vol}{"vfiler"}' src='$Config{"PATH_IMAGES"}/gauge_GREEN.jpg' width='$gauge_width_GREEN' /><img class='img_gauge' alt='$vfInfo{$vol}{"vfiler"}' src='$Config{"PATH_IMAGES"}/gauge_YELLOW.jpg' width='$gauge_width_YELLOW' />
DF_ROW_YELLOW
	}
	else
	{
 		print <<DF_ROW_GREEN;
	  <tr>
		<td class='$df_row' align='left'><a style='color:white' href='$Config{"AREF"}$link&cfg=$filer.cfg'><b>$temp</b></a></td>
		<td class='td_row_GAUGE'><img class='img_gauge' alt='$vfInfo{$vol}{"vfiler"}' src='$Config{"PATH_IMAGES"}/gauge_GREEN.jpg' width='$gauge_width_GREEN' />
DF_ROW_GREEN
	}

	# format numbers
	$dfTotal   = sprintf("%0.0f GB", $vfInfo{$vol}{Total});
	$dfUsed    = sprintf("%0.0f GB", $vfInfo{$vol}{Used});
	$dfAvail   = sprintf("%0.0f GB", $vfInfo{$vol}{Avail});
	$dfPercent = sprintf("% 2.0f",   $vfInfo{$vol}{Percent});

	print <<DF_ROW_REST;
		</td>
		<td class="$df_row" align="right">$dfTotal</td>
		<td class="$df_row" align="right">$dfUsed</td>
		<td class="$df_row" align="right">$dfAvail</td>
		<td class="$df_row" align="right">$dfPercent%</td>
	  </tr>
DF_ROW_REST

	return $df_switch;

}

#==============================================================================================================================

sub List_Volume($$$$)
{
	my $filer= shift;
	my $vFiler= shift;
	my $vol = shift;
	my $df_switch = shift;

  	my $df_status;
  	my $df_row;
  	my $gauge_width_GREEN;
  	my $gauge_width_YELLOW;
  	my $gauge_width_RED;
  	my $link;
  	my $warn;
  	my $danger;

	# set switches
	$df_switch = $df_switch == 1 ? 2 : 1;

	# cut excess
	$df_status = $vfInfo{$vol}{Percent} > 100 ? 100 : $vfInfo{$vol}{Percent};

	#printf STDOUT "Vol:  %-20s     ",$vol;

	#define levels
	($warn, $danger) = CalcWarnDanger($vfInfo{$vol}{Total});

	# define colors etc.
	if ( $df_status <= $warn )
	{
		$df_row = "td_row" . $df_switch . "_GREEN";
	  	$gauge_width_GREEN  = $df_status;
	  	$gauge_width_YELLOW = 0;
	  	$gauge_width_RED    = 0;;
	}
	elsif ( $df_status <= $danger )
	{
	  	$df_row = "td_row_YELLOW";
	  	$gauge_width_GREEN  = $warn;
	  	$gauge_width_YELLOW = $df_status - $warn;
	  	$gauge_width_RED    = 0;
	}
	else
	{
	  	$df_row = "td_row_RED";
	  	$gauge_width_GREEN  = $warn;
	  	$gauge_width_YELLOW = $danger - $warn;
	  	$gauge_width_RED    = $df_status - $danger;
	}

	$link = $filer;
	$link = $link . "_" . $vfInfo{$vol}{"fullname"};
	$link =~ tr/A-Z/a-z/;
	$link =~ s/vol//;
	$link =~ s/\///g;
	$link =~ s/\.snapshot//;
	$link =~ s/\.\.//;

	# print table row
	if ( $gauge_width_RED > 0 )
	{
		print <<DF_ROW_RED;
	  <tr>
		<td class='$df_row' align='left'><a href='$Config{"AREF"}$link&cfg=$filer.cfg'>$vfInfo{$vol}{"fullname"}</a></td>
		<td class='td_row_GAUGE'>
		<img class='img_gauge' alt='$vfInfo{$vol}{"fullname"}' src='$Config{"PATH_IMAGES"}/gauge_GREEN.jpg' width='$gauge_width_GREEN' /><img class='img_gauge' alt='$vfInfo{$vol}{"fullname"}' src='$Config{"PATH_IMAGES"}/gauge_YELLOW.jpg' width='$gauge_width_YELLOW' /><img class='img_gauge' alt='$vfInfo{$vol}{"fullname"}' src='$Config{"PATH_IMAGES"}/gauge_RED.jpg' width='$gauge_width_RED' />
DF_ROW_RED
	}
	elsif ( $gauge_width_YELLOW > 0 )
	{
 		print <<DF_ROW_YELLOW;
	  <tr>
		<td class='$df_row' align='left'><a href='$Config{"AREF"}$link&cfg=$filer.cfg'>$vfInfo{$vol}{"fullname"}</a></td>
		<td class='td_row_GAUGE'>
		<img class='img_gauge' alt='$vfInfo{$vol}{"fullname"}' src='$Config{"PATH_IMAGES"}/gauge_GREEN.jpg' width='$gauge_width_GREEN' /><img class='img_gauge' alt='$vfInfo{$vol}{"fullname"}' src='$Config{"PATH_IMAGES"}/gauge_YELLOW.jpg' width='$gauge_width_YELLOW' />
DF_ROW_YELLOW
	}
	else
	{
 		print <<DF_ROW_GREEN;
	  <tr>
		<td class='$df_row' align='left'><a href='$Config{"AREF"}$link&cfg=$filer.cfg'>$vfInfo{$vol}{"fullname"}</a></td>
		<td class='td_row_GAUGE'><img class='img_gauge' alt='$vfInfo{$vol}{"fullname"}' src='$Config{"PATH_IMAGES"}/gauge_GREEN.jpg' width='$gauge_width_GREEN' />
DF_ROW_GREEN
	}

	# format numbers
	$dfTotal   = sprintf("%0.0f GB", $vfInfo{$vol}{Total});
	$dfUsed    = sprintf("%0.0f GB", $vfInfo{$vol}{Used});
	$dfAvail   = sprintf("%0.0f GB", $vfInfo{$vol}{Avail});
	$dfPercent = sprintf("% 2.0f",   $vfInfo{$vol}{Percent});

	print <<DF_ROW_REST;
		</td>
		<td class="$df_row" align="right">$dfTotal</td>
		<td class="$df_row" align="right">$dfUsed</td>
		<td class="$df_row" align="right">$dfAvail</td>
		<td class="$df_row" align="right">$dfPercent%</td>
	  </tr>
DF_ROW_REST

	return $df_switch;

}

#==============================================================================================================================

sub CalcWarnDanger($)
{

    my $total  = shift;
    my $warn   = 0.00;
	my $danger = 0.00;
    my $div    = 0.00;

	#convert to float
	$total = sprintf("%f", $total);

    if ($total < 2500)
    {
		if ($total > 0)
		{
			$warn = log($total);
		}
	    $div = $warn/2.302585;

	    $warn   = int($Config{'WARNING'} + ($Config{'WARNINGINC'} * $div));
	    $danger = int($Config{'DANGER'}  + ($Config{'DANGERINC'}  * $div));
	}
	else
	{
		$warn   = 98;
		$danger = 99;
	}

	#printf STDOUT "Total: %04d - Warn: %s - Danger: %s\n", $total, $warn, $danger;

	return $warn, $danger;

}

#==============================================================================================================================

sub List_SnapshotInfo($)
{

    my $filer = shift;

    my $snapVolume;
    my $df_row;
    my $df_switch = 2;

    my @arSSdhms;
    my @diff;
    my $difference;

    # get and set date of tomorrow
    ($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = localtime;
    ($year, $mon, $day) = Add_Delta_Days($year,1+$mon,$day,1);
    # create ISO date
    $NWyyyy = 1900+$year;
    $NWmm = $mon;

    # get and set lowest date
    ($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = localtime;
    #($year, $mon, $day) = Add_Delta_Days($year,1+$mon,$day,-$offdays);
    ($year, $mon, $day) = Add_Delta_Days($year,1+$mon,$day,-15);

    # get info from filer
    @arSSmonth    = &snmpwalk($filer,OID_SSmonth);
    @arSSday      = &snmpwalk($filer,OID_SSday);
    @arSShour     = &snmpwalk($filer,OID_SShour);
    @arSSminute   = &snmpwalk($filer,OID_SSminute);
    @arSSname     = &snmpwalk($filer,OID_SSname);
    @arSSvolname  = &snmpwalk($filer,OID_SSvolname);

    # create hash based on OID
    foreach $desc (@arSSmonth)
    {
    	# printf "$desc\n";
        # convert to uppercase
        $desc =~ tr/a-z/A-Z/;
        # split of oid and month
        ($oid, $desc)  = split(':', $desc, 2);
        # split oid in 2
        ($oid1, $oid2) = split(/\./, $oid , 2);
        # reformat oid
        $oid = sprintf("%03d.%03d", $oid1, $oid2);
        $SSInfo{$oid}{datetime} = '';
        $SSmm = $desc;
        if ($SSmm <= $NWmm)
        {
    		# add year to month and put in hash
    	   	$SSInfo{$oid}{datetime} = sprintf("%04d-%02d", $NWyyyy, $SSmm);
    	}
    	else
    	{
    		# add year to month
    	   	$SSInfo{$oid}{datetime} = sprintf("%04d-%02d", $NWyyyy-1, $SSmm);
    	}
        #printf "$oid = $SSInfo{$oid}{datetime}\n";
    }

    # add days to hash
    foreach $desc (@arSSday)
    {
        # printf "$desc\n";
        # convert to uppercase
        $desc =~ tr/a-z/A-Z/;
        # split of oid and days
        ($oid, $desc) = split(':', $desc, 2);
        # split oid in 2
        ($oid1, $oid2) = split(/\./, $oid , 2);
        # reformat oid
        $oid = sprintf("%03d.%03d", $oid1, $oid2);

        if(!exists($SSInfo{$oid}{datetime}))
        {
       	   	$SSInfo{$oid}{datetime} = sprintf("%04d-%02d", $NWyyyy, $NWmm);
        }

        if(!defined($SSInfo{$oid}{datetime}))
        {
       	   	$SSInfo{$oid}{datetime} = sprintf("%04d-%02d", $NWyyyy, $NWmm);
        }

        if(length($SSInfo{$oid}{datetime}) == 0)
        {
       	   	$SSInfo{$oid}{datetime} = sprintf("%04d-%02d", $NWyyyy, $NWmm);
        }

    	# reformat day by adding 0 if day < 10 and put in hash
    	if ($desc && length($desc) > 0 )
    	{
           	$SSInfo{$oid}{datetime} = sprintf("%s-%02d", $SSInfo{$oid}{datetime}, $desc);
        }
        else
        {
           	$SSInfo{$oid}{datetime} = sprintf("%s-%02d", $SSInfo{$oid}{datetime}, 0);
        }
        # printf "$oid = $SSInfo{$oid}{datetime}\n";
    }

    # add hours to hash
    foreach $desc (@arSShour)
    {
        # printf "$desc\n";
        # convert to uppercase
        $desc =~ tr/a-z/A-Z/;
        # split of oid and days
        ($oid, $desc) = split(':', $desc, 2);
        # split oid in 2
        ($oid1, $oid2) = split(/\./, $oid , 2);
        # reformat oid
        $oid = sprintf("%03d.%03d", $oid1, $oid2);

        if(!exists($SSInfo{$oid}{datetime}))
        {
       	   	$SSInfo{$oid}{datetime} = sprintf("%04d-%02d", $NWyyyy, $NWmm);
        }

        if(!defined($SSInfo{$oid}{datetime}))
        {
       	   	$SSInfo{$oid}{datetime} = sprintf("%04d-%02d", $NWyyyy, $NWmm);
        }

        if(length($SSInfo{$oid}{datetime}) == 0)
        {
       	   	$SSInfo{$oid}{datetime} = sprintf("%04d-%02d", $NWyyyy, $NWmm);
        }

    	# reformat day by adding 0 if day < 10 and put in hash
        if($desc && length($desc) > 0 )
        {
       	   $SSInfo{$oid}{datetime} = sprintf("%s %02d", $SSInfo{$oid}{datetime}, $desc);
       	}
       	else
        {
       	   $SSInfo{$oid}{datetime} = sprintf("%s %02d", $SSInfo{$oid}{datetime}, 0);
       	}

        # printf "$oid = $SSInfo{$oid}{datetime}\n";
    }

    # add minutes to hash
    foreach $desc (@arSSminute)
    {
        # printf "$desc\n";
        # convert to uppercase
        $desc =~ tr/a-z/A-Z/;
        # split of oid and days
        ($oid, $desc) = split(':', $desc, 2);
        # split oid in 2
        ($oid1, $oid2) = split(/\./, $oid , 2);
        # reformat oid
        $oid = sprintf("%03d.%03d", $oid1, $oid2);

        if(!exists($SSInfo{$oid}{datetime}))
        {
       	   	$SSInfo{$oid}{datetime} = sprintf("%04d-%02d", $NWyyyy, $NWmm);
        }

        if(!defined($SSInfo{$oid}{datetime}))
        {
       	   	$SSInfo{$oid}{datetime} = sprintf("%04d-%02d", $NWyyyy, $NWmm);
        }

        if(length($SSInfo{$oid}{datetime}) == 0)
        {
       	   	$SSInfo{$oid}{datetime} = sprintf("%04d-%02d", $NWyyyy, $NWmm);
        }

    	# reformat day by adding 0 if day < 10 and put in hash
    	# reformat day by adding 0 if day < 10 and put in hash
        if($desc && length($desc) > 0 )
        {
           	$SSInfo{$oid}{datetime} = sprintf("%s:%02d:00", $SSInfo{$oid}{datetime}, $desc);
       	}
       	else
        {
           	$SSInfo{$oid}{datetime} = sprintf("%s:%02d:00", $SSInfo{$oid}{datetime}, 0);
       	}
        # printf "$oid = $SSInfo{$oid}{datetime}\n";
    }

    # add snapnames to hash
    foreach $desc (@arSSname)
    {
        # printf "$desc\n";
        # convert to uppercase
        $desc =~ tr/a-z/A-Z/;
        # split of oid and snapname
        ($oid, $desc) = split(':', $desc, 2);
        # split oid in 2
        ($oid1, $oid2) = split(/\./, $oid , 2);
        # reformat oid
        $oid = sprintf("%03d.%03d", $oid1, $oid2);
        # put in hash
       	$SSInfo{$oid}{snapname} = $desc;

        if(!exists($SSInfo{$oid}{snapname}))
        {
       	   	$SSInfo{$oid}{snapname} = " ";
        }

        if(!defined($SSInfo{$oid}{snapname}))
        {
       	   	$SSInfo{$oid}{snapname} = " ";
        }

        if(length($SSInfo{$oid}{snapname}) == 0)
        {
       	   	$SSInfo{$oid}{snapname} = " ";
        }
        # printf "$oid = $SSInfo{$oid}{snapname}\n";

    }

    # add volnames to hash
    foreach $desc (@arSSvolname)
    {
        # printf "$desc\n";
        # convert to uppercase
        $desc =~ tr/a-z/A-Z/;
        # split of oid and volume name
        ($oid, $desc) = split(':', $desc, 2);
        ($oid1, $oid2) = split(/\./, $oid , 2);
        # reformat oid
        $oid = sprintf("%03d.%03d", $oid1, $oid2);
        # put in hash
       	$SSInfo{$oid}{volname} = $desc;

        if(!exists($SSInfo{$oid}{volname}))
        {
       	   	$SSInfo{$oid}{volname} = " ";
        }

        if(!defined($SSInfo{$oid}{volname}))
        {
       	   	$SSInfo{$oid}{volname} = " ";
        }

        if(length($SSInfo{$oid}{volname}) == 0)
        {
       	   	$SSInfo{$oid}{volname} = " ";
        }
        # printf "$oid = $SSInfo{$oid}{volname}\n";
    }

    # push off "000.000"
    delete $SSInfo{"000.000"};

    # now traverse hash and find snapshots
    for $oid ( sort keys %SSInfo )
    {
     	# get date into array
     	# YYYY-MM-DD HH:MM:SS
     	# 0123456789012345678
  	    $temp = length($SSInfo{$oid}{datetime});
     	if (!$temp)
     	{
        	($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = localtime();
        	$SSInfo{$oid}{datetime} = sprintf("%04d-%02d-%02d %02d:%02d:%02d", 1900+$year, $mon+1, $day, $hour, $min, $sec);
        }
        elsif ($temp < 19)
     	{
     	    $temp = 19 - $temp;
     	    $SSInfo{$oid}{datetime} = sprintf("%s%0${temp}d", $SSInfo{$oid}{datetime}, 0);
     	}
     	$arSSdhms[0] = substr($SSInfo{$oid}{datetime}, 0,4);
     	$arSSdhms[1] = substr($SSInfo{$oid}{datetime}, 5,2);
     	$arSSdhms[2] = substr($SSInfo{$oid}{datetime}, 8,2);
     	$arSSdhms[3] = substr($SSInfo{$oid}{datetime},11,2);
     	$arSSdhms[4] = substr($SSInfo{$oid}{datetime},14,2);
     	$arSSdhms[5] = substr($SSInfo{$oid}{datetime},17,2);

        ($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = localtime;

     	@diff = Delta_DHMS($arSSdhms[0], $arSSdhms[1], $arSSdhms[2], $arSSdhms[3], $arSSdhms[4], $arSSdhms[5], 1900+$year, $mon+1, $day, $hour, $min, $sec);

        for my $indx (0 .. 3)
        {
            if (!exists $diff[$indx])
            {
                push (@diff, 0);
            }
            elsif (!defined $diff[$indx])
            {
                $diff[$indx] = 0;
            }
            elsif (length($diff[$indx]) == 0)
            {
                $diff[$indx] = 0;
            }
        }

        if(!exists $SSInfo{$oid}{volname})
        {
            $SSInfo{$oid}{volname} = "Not Found";
        }
        elsif (!defined $SSInfo{$oid}{volname})
        {
            $SSInfo{$oid}{volname} = "Not Found";
        }
        elsif (length($SSInfo{$oid}{volname}) == 0)
        {
            $SSInfo{$oid}{volname} = "Not Found";
        }


        if(!exists $SSInfo{$oid}{snapname})
        {
            $SSInfo{$oid}{snapname} = "Not Found";
        }
        elsif (!defined $SSInfo{$oid}{snapname})
        {
            $SSInfo{$oid}{snapname} = "Not Found";
        }
        elsif (length($SSInfo{$oid}{snapname}) == 0)
        {
            $SSInfo{$oid}{snapname} = "Not Found";
        }

        if(!exists $SSInfo{$oid}{datetime})
        {
            $SSInfo{$oid}{datetime} = "1900-01-01 00:00:00";
        }
        elsif (!defined $SSInfo{$oid}{datetime})
        {
            $SSInfo{$oid}{datetime} = "1900-01-01 00:00:00";
        }
        elsif (length($SSInfo{$oid}{datetime}) == 0)
        {
            $SSInfo{$oid}{datetime} = "1900-01-01 00:00:00";
        }

        #print "Difference: $diff[0] days, $diff[1]:$diff[2]:$diff[3]\n";

     	# set switches
    	$df_switch = $df_switch == 1 ? 2 : 1;

    	if ($diff[0] < $Config{'SSLAG'})
    	{
			$df_row = "td_row" . $df_switch . "_GREEN";
    	}
    	else
    	{
    	  	$df_row = "td_row_RED";
        }

        if ($diff[0] > 1)
        {
            $difference = sprintf("%d days, %02d:%02d:%02d", $diff[0], $diff[1],$diff[2],$diff[3]);
            print <<SS_DF_ROW_M;
        	  <tr>
    	        <td class='$df_row' align='left'  >$SSInfo{$oid}{volname}</td>
    	        <td class='$df_row' align='left'  >$SSInfo{$oid}{snapname}</td>
    	        <td class='$df_row' align='center'>$SSInfo{$oid}{datetime}</td>
    	        <td class='$df_row' align='right' >$difference</td>
              </tr>
SS_DF_ROW_M
        }
        elsif ($diff[0] == 1)
        {
            $difference = sprintf("%d day , %02d:%02d:%02d", $diff[0], $diff[1],$diff[2],$diff[3]);
            print <<SS_DF_ROW_1;
        	  <tr>
    	        <td class='$df_row' align='left'  >$SSInfo{$oid}{volname}</td>
    	        <td class='$df_row' align='left'  >$SSInfo{$oid}{snapname}</td>
    	        <td class='$df_row' align='center'>$SSInfo{$oid}{datetime}</td>
    	        <td class='$df_row' align='right' >$difference</td>
              </tr>
SS_DF_ROW_1
        }
        else
        {
            $difference = sprintf("%02d:%02d:%02d", $diff[1],$diff[2],$diff[3]);
            print <<SS_DF_ROW_0;
        	  <tr>
    	        <td class='$df_row' align='left'  >$SSInfo{$oid}{volname}</td>
    	        <td class='$df_row' align='left'  >$SSInfo{$oid}{snapname}</td>
    	        <td class='$df_row' align='center'>$SSInfo{$oid}{datetime}</td>
    	        <td class='$df_row' align='right' >$difference</td>
              </tr>
SS_DF_ROW_0
        }

    }
}

#==============================================================================================================================

sub List_SnapmirrorInfo($)
{

    my $filer = shift;

    my $df_row;
    my $df_switch = 2;

    my $lag;

    # get snmp info
    @arSMsrc    = &snmpwalk($filer,OID_SMsrc);
    @arSMdst    = &snmpwalk($filer,OID_SMdst);
    @arSMstatus = &snmpwalk($filer,OID_SMstatus);
    @arSMlag    = &snmpwalk($filer,OID_SMlag);

    # create hash based on OID
    foreach $desc (@arSMsrc)
    {
    	#printf "$desc\n";
        # convert to uppercase
        $desc =~ tr/a-z/A-Z/;
        # split of oid and month
        ($oid, $desc)  = split(':', $desc, 2);
        # reformat oid
        $oid = sprintf("%03d.000", $oid);
     	$SMInfo{$oid}{src} = $desc;
        #printf "$oid = $SMInfo{$oid}{src}\n";
    }

    # add dst to hash
    foreach $desc (@arSMdst)
    {
        #printf "$desc\n";
        # convert to uppercase
        $desc =~ tr/a-z/A-Z/;
        # split of oid and days
        ($oid, $desc) = split(':', $desc, 2);
        # reformat oid
        $oid = sprintf("%03d.000", $oid);
    	# reformat day by adding 0 if day < 10 and put in hash
       	$SMInfo{$oid}{dst} = $desc;
        #printf "$oid = $SMInfo{$oid}{dst}\n";
    }

    # add status to hash
    foreach $desc (@arSMstatus)
    {
        #printf "$desc\n";
        # convert to uppercase
        $desc =~ tr/a-z/A-Z/;
        # split of oid and snapname
        ($oid, $desc) = split(':', $desc, 2);
        # reformat oid
        $oid = sprintf("%03d.000", $oid);
        # put in hash
       	$SMInfo{$oid}{status} = $SMstatus[$desc-1];
        #printf "$oid = $SMInfo{$oid}{status}\n";
    }

    # add lag to hash
    foreach $desc (@arSMlag)
    {
        #printf "$desc\n";
        # convert to uppercase
        $desc =~ tr/a-z/A-Z/;
        # split of oid and volume name
        ($oid, $desc) = split(':', $desc, 2);
        # reformat oid
        $oid = sprintf("%03d.000", $oid);
        # put in hash
        $desc = lc($desc);
       	$SMInfo{$oid}{lag} = $desc;
        #printf "$oid = $SMInfo{$oid}{lag}\n";
    }

    # push off "000.000"
    delete $SMInfo{"000.000"};

    # now traverse hash and find snapshots
    for $oid ( sort keys %SMInfo )
    {
     	# set switches
    	$df_switch = $df_switch == 1 ? 2 : 1;

        $lag = 0;
        $temp = $SMInfo{$oid}{lag};
        $temp =~ s/ day(s)?//gi;
        if ($temp =~ /^(.*),(.*)$/)
        {
            $lag = $1 *24;
            $temp = $2;
            #printf "D1 = $1 - D2 = $2\n";
        }
        # trim leading and trailing spaces
        $temp =~ s/^\ *([A-Z,a-z,0-9]*)\ *$/$1/g;
        $temp =~ /^(.*):(.*):(.*)$/;
        $lag += $1;
        #printf "Temp = $temp - D1 = $1 - D2 = $2 - D3 = $3 - lag = $lag\n";

    	if ($lag < $Config{'SMLAG'})
    	{
			$df_row = "td_row" . $df_switch . "_GREEN";
    	}
    	else
    	{

    	  	$df_row = "td_row_RED";
        }
        print <<SM_DF_ROW;
    	  <tr>
	        <td class='$df_row' align='left'  >$SMInfo{$oid}{src}</td>
	        <td class='$df_row' align='left'  >$SMInfo{$oid}{dst}</td>
	        <td class='$df_row' align='center'>$SMInfo{$oid}{status}</td>
	        <td class='$df_row' align='right' >$SMInfo{$oid}{lag}</td>
          </tr>
SM_DF_ROW
    }
}

#==============================================================================================================================

sub List_SnapvaultInfo($)
{

    my $filer = shift;

    my $lag;

    my $df_row;
    my $df_switch = 2;

   # get snmp info
    @arSVsrc    = &snmpwalk($filer,OID_SVsrc);
    @arSVdst    = &snmpwalk($filer,OID_SVdst);
    @arSVstatus = &snmpwalk($filer,OID_SVstatus);
    @arSVlag    = &snmpwalk($filer,OID_SVlag);

    # create hash based on OID
    foreach $desc (@arSVsrc)
    {
    	#printf "$desc\n";
        # convert to uppercase
        $desc =~ tr/a-z/A-Z/;
        # split of oid and month
        ($oid, $desc)  = split(':', $desc, 2);
        # reformat oid
        $oid = sprintf("%03d.000", $oid);
     	$SVInfo{$oid}{src} = $desc;
        #printf "$oid = $SVInfo{$oid}{src}\n";
    }

    # add dst to hash
    foreach $desc (@arSVdst)
    {
        #printf "$desc\n";
        # convert to uppercase
        $desc =~ tr/a-z/A-Z/;
        # split of oid and days
        ($oid, $desc) = split(':', $desc, 2);
        # reformat oid
        $oid = sprintf("%03d.000", $oid);
    	# reformat day by adding 0 if day < 10 and put in hash
       	$SVInfo{$oid}{dst} = $desc;
        #printf "$oid = $SVInfo{$oid}{dst}\n";
    }

    # add status to hash
    foreach $desc (@arSVstatus)
    {
        #printf "$desc\n";
        # convert to uppercase
        $desc =~ tr/a-z/A-Z/;
        # split of oid and snapname
        ($oid, $desc) = split(':', $desc, 2);
        # reformat oid
        $oid = sprintf("%03d.000", $oid);
        # put in hash
       	$SVInfo{$oid}{status} = $SVstatus[$desc-1];
        #printf "$oid = $SVInfo{$oid}{status}\n";
    }

    # add lag to hash
    foreach $desc (@arSVlag)
    {
        #printf "$desc\n";
        # convert to uppercase
        $desc =~ tr/a-z/A-Z/;
        # split of oid and volume name
        ($oid, $desc) = split(':', $desc, 2);
        # reformat oid
        $oid = sprintf("%03d.000", $oid);
        # put in hash
        $desc = lc($desc);
       	$SVInfo{$oid}{lag} = $desc;
        #printf "$oid = $SVInfo{$oid}{lag}\n";
    }

    # push off "000.000"
    delete $SVInfo{"000.000"};

    # now traverse hash and find snapshots
    for $oid ( sort keys %SVInfo )
    {
     	# set switches
    	$df_switch = $df_switch == 1 ? 2 : 1;

        $lag = 0;
        $temp = $SVInfo{$oid}{lag};
        $temp =~ s/ day(s)?//gi;
        if ($temp =~ /^(.*),(.*)$/)
        {
            $lag = $1 *24;
            $temp = $2;
            #printf "D1 = $1 - D2 = $2\n";
        }
        # trim leading and trailing spaces
        $temp =~ s/^\ *([A-Z,a-z,0-9]*)\ *$/$1/g;
        $temp =~ /^(.*):(.*):(.*)$/;
        $lag += $1;
        #printf "Temp = $temp - D1 = $1 - D2 = $2 - D3 = $3 - lag = $lag\n";

    	if ($lag < $Config{'SVLAG'})
    	{
			$df_row = "td_row" . $df_switch . "_GREEN";
    	}
    	else
    	{

    	  	$df_row = "td_row_RED";
        }
        print <<SV_DF_ROW;
    	  <tr>
	        <td class='$df_row' align='left'  >$SVInfo{$oid}{src}</td>
	        <td class='$df_row' align='left'  >$SVInfo{$oid}{dst}</td>
	        <td class='$df_row' align='center'>$SVInfo{$oid}{status}</td>
	        <td class='$df_row' align='right' >$SVInfo{$oid}{lag}</td>
          </tr>
SV_DF_ROW
    }
}

#==============================================================================================================================

sub SendMail ($$)
{
	my $smtp;
	my $subject = shift;
	my $body    = shift;

	($sec, $min, $hour, $day, $mon, $year, $wday, $yday, $isdst) = localtime();
	$now = sprintf("%04d-%02d-%02d %02d:%02d:%02d", 1900+$year, $mon+1, $day, $hour, $min, $sec);

	if ($from)
	{
		if ( $smtp = Net::SMTP->new($Config{'SMTPSERVER'}, Timeout => 15, Debug => DEBUG) )
		{
		    # Calculate the time it takes to send an email
		    $smtp->mail($from);
		    $smtp->to($Config{'TO'});
		    $smtp->data();
		    $smtp->datasend("To: $Config{'TO'}\n");
		    $smtp->datasend("Subject: $subject\n");
		    $smtp->datasend("\n");
		    $smtp->datasend("$body\n");
		    $smtp->dataend();
		    $smtp->quit;

		    return 1;
		}
		else
		{
			print STDOUT "$now : Error sending email\n" if $LOG;
			return 0;
		}
	}
	else
	{
		print STDOUT "$now : No From and/or bad To ($Config{'TO'}) emailadress\n" if $LOG;
		return 0;
	}
}

#==============================================================================================================================

sub LoadAlerts ()
{
	my $in = sprintf("%s.Alerts", $fname);
	my @cols;
	my $col;
	my $filer;

	#%Alerts = ("Filer" => { Fans => 0, PowerSupply => 0, Disks => 0, Battery => 0, Global => 0});

	if (open IN, "<$in")
	{
		while (<IN>)
		{
		  	next if /^\#/;
		  	chomp;
		  	next unless $_; # skip empty lines
		  	@cols = split / - /;

		  	# get first element = filer
		  	$filer = $cols[0];
		  	# remove it
		  	shift(@cols);

			# do all the rest
		  	foreach $col (@cols)
		  	{
		  		# print STDERR "col = $col\n" if DEBUG;
			  	$col =~ s/\=/___EQ___/; # replace first equal sign
			  	my ($key, $value) = split(/___EQ___/, $col);
			  	$Alerts{$filer}{$key} = sprintf("%d", $value);
		  		#print STDERR "$key = $Alerts{$filer}{$key}\n" if DEBUG;
			}
		}
		close IN;
	}
}

#==============================================================================================================================

sub SaveAlerts ()
{
	my $out = sprintf("%s.Alerts", $fname);

	#%Alerts = ("Filer" => { Fans => 0, PowerSupply => 0, Disks => 0, Battery => 0, Global => 0});

	if(open OUT, ">$out")
	{
	    for $key ( sort keys %Alerts )
	    {
	       	print OUT  $key . " - ";
	       	print OUT  "Fans=" . $Alerts{$key}{Fans} . " - ";
	       	print OUT  "PowerSupply=" . $Alerts{$key}{PowerSupply} . " - ";
	       	print OUT  "Disks=" . $Alerts{$key}{Disks} . " - ";
	       	print OUT  "Battery=" . $Alerts{$key}{Battery} . " - ";
	       	print OUT  "Global=" . $Alerts{$key}{Global};
	       	print OUT  "\n";
	    }
		close OUT;
	}
}
